/* --------------------------------------------------------------------------------
 * Example sketch/program showing how to read data from a PICC to serial.
 * This is modified from a MFRC522 library example; 
 * 
 * @license Released into the public domain.
*/
#include <SPI.h>
#include <MFRC522.h>

constexpr uint8_t RST_PIN = 9;
constexpr uint8_t SS_PIN = 10;
MFRC522 mfrc522(SS_PIN, RST_PIN);  // Create MFRC522 instance

MFRC522::MIFARE_Key keyA;

void setup() {
	Serial.begin(9600);		// Initialize serial communications with the PC
	while (!Serial);		// Do nothing if no serial port is opened 
	SPI.begin();			// Init SPI bus
	mfrc522.PCD_Init();		// Init MFRC522
	mfrc522.PCD_DumpVersionToSerial();	// Show details of PCD - MFRC522 Card Reader
	Serial.println(F("Scan PICC to see UID, SAK, type, and data blocks..."));
  Serial.println();
    // Prepare the key A
    // using FFFFFFFFFFFFh which is the default at chip delivery from the factory
    for (byte i = 0; i < 6; i++) {
        keyA.keyByte[i] = 0xFF;
    }
}

/*  Main loop */
void loop() {
    // Look for new cards
    if ( ! mfrc522.PICC_IsNewCardPresent())
        return;

    // Select one of the cards
    if ( ! mfrc522.PICC_ReadCardSerial())
        return;

    // Show some details of the PICC (that is: the tag/card)
    Serial.print(F("Card UID:")); dump_byte_array(mfrc522.uid.uidByte, mfrc522.uid.size);
    Serial.println();

    // here comes your fancy code ===============================================================
  
error:
    // END OF Example =============================================================
    // Halt PICC
    mfrc522.PICC_HaltA();
    // Stop encryption on PCD, always do this if authentication was used !!!
    mfrc522.PCD_StopCrypto1();
} /* loop ends here */

/**
 * Helper routines 
 */    
void myErrorPrint(char *msg, MFRC522::StatusCode status) {
        Serial.print(msg);
        Serial.println(mfrc522.GetStatusCodeName(status));
 }

void  SerialBlockPrint(char *msg, byte addr, byte *buf) {
    Serial.print(msg); Serial.print(addr);
    Serial.println(F(" ..."));
    dump_byte_array(buf, 16); Serial.println();  
}

void dump_byte_array(byte *buffer, byte bufferSize) {
    for (byte i = 0; i < bufferSize; i++) {
        Serial.print(buffer[i] < 0x10 ? " 0" : " ");
        Serial.print(buffer[i], HEX);
    }
}

/**
 * Use this routine to continue the program after an error occurred.
 * You may have to reauthenticate again if the error occorred in
 * an authenticated state.
 */
bool ErrorRecover(){
      MFRC522::StatusCode status;
      byte bufferATQA[10];
      byte bufferSize[10];
      mfrc522.PICC_HaltA();
      // Stop encryption on PCD, always do this if authentication was used !!!
      mfrc522.PCD_StopCrypto1();
      delay(500); 
      if ( status = mfrc522.PICC_WakeupA(bufferATQA, bufferSize)){ //wake up PICC which was previously Halted
        Serial.println(mfrc522.GetStatusCodeName(status));
        return false;
      }
      delay(500);   
      // Select the card again
      if ( ! mfrc522.PICC_ReadCardSerial()){
        return false;
      }
      Serial.println("Recovered");
      return true;
}
